@@ -624,12 +624,17 @@ pseudo_fix_path(const char *base, const char *path, size_t rootlen, size_t basel
 	char *newpath;
 	char *current;
 	char *effective_root;
+	int trailing_slash = 0;
 	
 	if (!path) {
 		pseudo_diag("can't fix empty path.\n");
 		return 0;
 	}
 	pathlen = strlen(path);
+	/* a trailing slash has special meaning */
+	if (pathlen > 0 && path[pathlen - 1] == '/') {
+		trailing_slash = 1;
+	}
 	newpathlen = pathlen;
         /* If the path starts with /, we don't care about base, UNLESS
          * rootlen is non-zero, in which case we're doing a chroot thing
@@ -667,7 +672,7 @@ pseudo_fix_path(const char *base, const char *path, size_t rootlen, size_t basel
 	 */
 	if (pseudo_append_elements(&newpath, &effective_root, &newpathlen, &current, path, pathlen, leave_last) != -1) {
 		--current;
-		if (*current == '/' && current > effective_root) {
+		if (*current == '/' && current > effective_root && !trailing_slash) {
 			*current = '\0';
 		}
 		pseudo_debug(PDBGF_PATH, "%s + %s => <%s>\n",
@@ -1264,7 +1269,7 @@ FILE *pseudo_host_etc_group_file = &pseudo_fake_group_file;
 #endif
 
 int
-pseudo_etc_file(const char *file, char *realname, int flags, char *search_dirs[], int dircount) {
+pseudo_etc_file(const char *file, char *realname, int flags, const char **search_dirs, int dircount) {
 	char filename[pseudo_path_max()];
 	int rc = -1;
 
@@ -1280,9 +1285,16 @@ pseudo_etc_file(const char *file, char *realname, int flags, char *search_dirs[]
 		return -1;
 	}
 	for (i = 0; i < dircount; ++i) {
-		char *s = search_dirs[i];
+		const char *s = search_dirs[i];
+		/* we used to pass in some paths as NULL when unset,
+		 * so we skipped those. Now NULL entries don't get
+		 * put in, so the only NULL should be the sentinel
+		 * value, and this should never get hit.
+		 *
+		 * "should" is not comforting to me.
+		 */
 		if (!s)
-			continue;
+			break;
 #if PSEUDO_PORT_DARWIN
 		/* special magic: empty string implies our emulation
 		 * of the passwd/group files.
